"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var deep_extends_1 = require("./deep-extends");
describe('deepExtends', function () {
    describe('basic', function () {
        it('should throw with bad args', function () {
            expect(function () { return deep_extends_1.deepExtends(); }).toThrowError('Invalid arguments: [] must provide a list of json values.');
            expect(function () { return deep_extends_1.deepExtends('someString'); }).toThrowError('Invalid arguments: [someString] must provide a list of json values.');
        });
        it('should work with just one', function () {
            var one = { rising: 'down' };
            expect(deep_extends_1.deepExtends(one)).toEqual({ rising: 'down' });
        });
        it('should ignore undefined', function () {
            var one = { rising: 'down' };
            var two = undefined;
            expect(deep_extends_1.deepExtends(one, two)).toEqual({ rising: 'down' });
        });
        it('should ignore null', function () {
            var one = { rising: 'down' };
            var two = null;
            expect(deep_extends_1.deepExtends(one, two)).toEqual({ rising: 'down' });
        });
        it('can deepExtends on 1 level', function () {
            var one = { home: 'grown!' };
            var two = { wake: 'up!' };
            expect(deep_extends_1.deepExtends(one, two)).toEqual({
                home: 'grown!',
                wake: 'up!'
            });
        });
        it('can do nested deepExtends', function () {
            var one = { member: { name: 'Captain Kirk Douglas' } };
            var two = { member: { role: 'guitar' } };
            expect(deep_extends_1.deepExtends(one, two)).toEqual({
                member: { name: 'Captain Kirk Douglas', role: 'guitar' }
            });
        });
        it('does not mutate objects', function () {
            var one = { doe: ['a', 'deer'] };
            var two = { a: ['female', 'deer'] };
            var three = { ray: 'a drop of golden sun' };
            var d = deep_extends_1.deepExtends({}, one, two, three);
            expect(one).toEqual({ doe: ['a', 'deer'] });
            expect(two).toEqual({ a: ['female', 'deer'] });
            expect(three).toEqual({ ray: 'a drop of golden sun' });
        });
    });
    describe('arrays', function () {
        it('works basic', function () {
            expect(deep_extends_1.deepExtends({ numbers: [1, 2, 3] }, { numbers: [2, 3] })).toEqual({ numbers: [2, 3] });
        });
        it('works for array notation', function () {
            var one = {
                albums: [{
                        name: 'organix',
                        release: 1993,
                        tracks: [
                            {
                                name: 'the roots is comin',
                                track: 1,
                                length: '1:15'
                            },
                            {
                                name: 'pass the popcorn',
                                track: 2,
                                length: '1:15'
                            },
                            {
                                name: 'the anti circle',
                                track: 3,
                                length: '3:46'
                            },
                            {
                                name: 'writer\'s block',
                                track: 4,
                                length: '1:42'
                            }
                        ]
                    }],
                singles: 8
            };
            var two = {
                'albums.0.tracks.2.length': '1:20'
            };
            var three = {
                'albums.0.tracks.3.length': '3:14'
            };
            expect(deep_extends_1.deepExtends(one, two, three)).toEqual({
                albums: [{
                        name: 'organix',
                        release: 1993,
                        tracks: [
                            {
                                name: 'the roots is comin',
                                track: 1,
                                length: '1:15'
                            },
                            {
                                name: 'pass the popcorn',
                                track: 2,
                                length: '1:15'
                            },
                            {
                                name: 'the anti circle',
                                track: 3,
                                length: '1:20'
                            },
                            {
                                name: 'writer\'s block',
                                track: 4,
                                length: '3:14'
                            }
                        ]
                    }],
                singles: 8
            });
        });
        it('works for append', function () {
            var one = {
                albums: [
                    {
                        name: 'organix'
                    },
                    {
                        name: 'the tipping point'
                    }
                ]
            };
            var two = { 'albums.[append]': { name: 'game theory' } };
            expect(deep_extends_1.deepExtends(one, two)).toEqual({
                albums: [
                    {
                        name: 'organix'
                    },
                    {
                        name: 'the tipping point'
                    },
                    {
                        name: 'game theory'
                    }
                ]
            });
            expect(deep_extends_1.deepExtends(one, two, { 'albums.0.name': 'rising down' })).toEqual({
                albums: [
                    {
                        name: 'rising down'
                    },
                    {
                        name: 'the tipping point'
                    },
                    {
                        name: 'game theory'
                    }
                ]
            });
        });
    });
    describe('more', function () {
        it('works', function () {
            var obj1 = {
                money: 1,
                bag: 2,
                nice: {
                    a: 1,
                    b: [],
                    c: { an: 123, ice: 321, bag: 1 }
                },
                swag: {
                    diamond: ['collar'],
                },
                pockets: { ice: 3 },
                f: ['bag']
            };
            var obj2 = {
                bag: 3,
                nice: null,
                'swag.diamond.0': 'collar',
                'swag.diamond.1': 'teeth',
                pockets: { need: 1, an: 2 }
            };
            expect(deep_extends_1.deepExtends(obj1, obj2)).toEqual({
                money: 1,
                bag: 3,
                nice: null,
                swag: {
                    diamond: ['collar', 'teeth']
                },
                pockets: { need: 1, an: 2, ice: 3 },
                f: ['bag']
            });
            var obj3 = {
                pockets: {
                    have: 'an', bag: { contents: 'ice' }
                }
            };
            expect(deep_extends_1.deepExtends(obj1, obj2, obj3)).toEqual({
                money: 1,
                bag: 3,
                nice: null,
                swag: {
                    diamond: ['collar', 'teeth']
                },
                pockets: { need: 1, an: 2, ice: 3, have: 'an', bag: { contents: 'ice' } },
                f: ['bag']
            });
            var obj4 = {
                'pockets.bag.contents': 'ice-cream'
            };
            expect(deep_extends_1.deepExtends(obj1, obj2, obj3, obj4)).toEqual({
                money: 1,
                bag: 3,
                nice: null,
                swag: {
                    diamond: ['collar', 'teeth']
                },
                pockets: { need: 1, an: 2, ice: 3, have: 'an', bag: { contents: 'ice-cream' } },
                f: ['bag']
            });
        });
        it('kitchen sink', function () {
            var obj1 = {
                money: 1,
                bag: 2,
                nice: {
                    a: 1,
                    b: [],
                    c: { an: 123, ice: 321, bag: 1 }
                },
                swag: {
                    diamond: ['collar'],
                },
                pockets: { ice: 3 },
                f: ['bag']
            };
            var obj2 = {
                bag: 3,
                nice: null,
                pockets: { need: 1, an: 2 }
            };
            expect(deep_extends_1.deepExtends(obj1, obj2)).toEqual({
                money: 1,
                bag: 3,
                nice: null,
                swag: {
                    'diamond': ['collar']
                },
                pockets: { need: 1, an: 2, ice: 3 },
                f: ['bag']
            });
        });
    });
});
